import { Role } from '@modules/ad/core/domain/ad.types';
import { Candidate } from '../../../types/algorithm.types';
import { Selector } from '../algorithm.abstract';
import { AdReadModel } from '@modules/ad/infrastructure/ad.repository';

export class PassengerOrientedSelector extends Selector {
  select = async (): Promise<Candidate[]> => {
    const queryStringRoles: QueryStringRole[] = [];
    if (this.query.driver)
      queryStringRoles.push({
        query: this.asDriverQueryString(),
        role: Role.DRIVER,
      });
    if (this.query.passenger)
      queryStringRoles.push({
        query: this.asPassengerQueryString(),
        role: Role.PASSENGER,
      });

    return (
      await Promise.all(
        queryStringRoles.map(async (queryStringRole: QueryStringRole) => ({
          ads: await this.repository.getCandidates(queryStringRole.query),
          role: queryStringRole.role,
        })),
      )
    )
      .map((adsRole) =>
        adsRole.ads.map(
          (adReadModel: AdReadModel) =>
            <Candidate>{
              ad: {
                id: adReadModel.uuid,
              },
              role: adsRole.role,
            },
        ),
      )
      .flat();
  };

  private asPassengerQueryString = (): string => `SELECT
        ad.uuid,driver,passenger,frequency,public.st_astext(matcher.ad.waypoints) as waypoints,
        "fromDate","toDate",
        "seatsProposed","seatsRequested",
        strict,
        "driverDuration","driverDistance",
        "passengerDuration","passengerDistance",
        "fwdAzimuth","backAzimuth",
        si.day,si.time,si.margin
        FROM ad LEFT JOIN schedule_item si ON ad.uuid = si."adUuid"
        WHERE driver=True`;

  private asDriverQueryString = (): string => `SELECT
        ad.uuid,driver,passenger,frequency,public.st_astext(matcher.ad.waypoints) as waypoints,
        "fromDate","toDate",
        "seatsProposed","seatsRequested",
        strict,
        "driverDuration","driverDistance",
        "passengerDuration","passengerDistance",
        "fwdAzimuth","backAzimuth",
        si.day,si.time,si.margin
        FROM ad LEFT JOIN schedule_item si ON ad.uuid = si."adUuid"
        WHERE passenger=True`;

  // await this.repository.getCandidates(this.query);
}

export type QueryStringRole = {
  query: string;
  role: Role;
};
